# Copyright (c) 2014 NetApp, Inc.
# All rights reserved.

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.forms import ValidationError
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.views.decorators.debug import sensitive_variables
from horizon import exceptions
from horizon import forms
from horizon import messages

from manila_ui.api import manila


class Create(forms.SelfHandlingForm):
    name = forms.CharField(max_length="255", label=_("Name"))
    dns_ip = forms.CharField(max_length="15", label=_("DNS IP"))
    ou = forms.CharField(
        max_length="255",
        label=_("Organizational Unit"),
        required=False)
    server = forms.CharField(max_length="255", label=_("Server"))
    domain = forms.CharField(max_length="255", label=_("Domain"))
    user = forms.CharField(max_length="255", label=_("User"), required=False)
    password = forms.CharField(
        label=_("Password"),
        widget=forms.PasswordInput(render_value=False),
        required=False)
    confirm_password = forms.CharField(
        label=_("Confirm Password"),
        widget=forms.PasswordInput(render_value=False),
        required=False)
    type = forms.ChoiceField(choices=(("", ""),
                                      ("active_directory", "Active Directory"),
                                      ("ldap", "LDAP"),
                                      ("kerberos", "Kerberos")),
                             label=_("Type"))
    description = forms.CharField(widget=forms.Textarea,
                                  label=_("Description"), required=False)

    def clean(self):
        '''Check to make sure password fields match.'''
        data = super(forms.Form, self).clean()
        if 'password' in data:
            if data['password'] != data.get('confirm_password', None):
                raise ValidationError(_('Passwords do not match.'))
        return data

    @sensitive_variables('data')
    def handle(self, request, data):
        try:
            data.pop('confirm_password')
            security_service = manila.security_service_create(
                request, **data)
            messages.success(request, _('Successfully created security '
                                        'service: %s') % data['name'])
            return security_service
        except Exception:
            exceptions.handle(request,
                              _('Unable to create security service.'))
            return False


class Update(forms.SelfHandlingForm):
    name = forms.CharField(max_length="255", label=_("Share Name"))
    dns_ip = forms.CharField(
        max_length="15", label=_("DNS IP"), required=False)
    ou = forms.CharField(
        max_length="255",
        label=_("Organizational Unit"),
        required=False)
    server = forms.CharField(
        max_length="255", label=_("Server"), required=False)
    domain = forms.CharField(
        max_length="255", label=_("Domain"), required=False)
    user = forms.CharField(
        max_length="255", label=_("User"), required=False)
    password = forms.CharField(
        label=_("Password"),
        widget=forms.PasswordInput(render_value=False),
        required=False)
    confirm_password = forms.CharField(
        label=_("Confirm Password"),
        widget=forms.PasswordInput(render_value=False),
        required=False)
    description = forms.CharField(
        widget=forms.Textarea, label=_("Description"), required=False)

    def clean(self):
        '''Check to make sure password fields match.'''
        cleaned_data = super(forms.Form, self).clean()
        password = self.cleaned_data.get('password')
        confirm_password = self.cleaned_data.get('confirm_password')
        if password != confirm_password:
            raise ValidationError(_('Passwords do not match.'))
        return cleaned_data

    @sensitive_variables('data')
    def handle(self, request, data):
        sec_service_id = self.initial['sec_service_id']
        try:
            manila.security_service_update(
                request, sec_service_id,
                dns_ip=data['dns_ip'],
                ou=data['ou'],
                server=data['server'],
                domain=data['domain'],
                password=data.get('password') or None,
                user=data['user'],
                name=data['name'],
                description=data['description'])

            message = _('Successfully updated security service '
                        '"%s"') % data['name']
            messages.success(request, message)
            return True
        except Exception:
            redirect = reverse("horizon:project:security_services:index")
            exceptions.handle(request,
                              _('Unable to update security service.'),
                              redirect=redirect)



# Copyright (c) 2016 Mirantis, Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy

from horizon import messages
from horizon import tables

from manila_ui.api import manila
from manila_ui.dashboards.project.shares.replicas import (
    tables as replica_tables)


DELETABLE_STATUSES = ("error", "available")


class UpdateReplicaRow(tables.Row):
    ajax = True

    def get_data(self, request, replica_id):
        replica = manila.share_replica_get(request, replica_id)
        return replica


class ResyncReplica(tables.LinkAction):
    name = "resync_replica"
    verbose_name = _("Resync replica")
    url = "horizon:admin:shares:resync_replica"
    classes = ("ajax-modal", "btn-create")
    policy_rules = (("share_replica", "share_replica:resync"),)

    def allowed(self, request, replica=None):
        return True

    def get_policy_target(self, request, datum=None):
        return {"project_id": getattr(datum, "project_id", None)}

    def get_link_url(self, replica):
        return reverse(self.url, args=(replica.id,))


class ResetReplicaState(tables.LinkAction):
    name = "reset_replica_state"
    verbose_name = _("Reset replica state")
    url = "horizon:admin:shares:reset_replica_state"
    classes = ("ajax-modal", "btn-create")
    policy_rules = (("share_replica", "share_replica:reset_replica_state"),)

    def allowed(self, request, replica=None):
        return True

    def get_policy_target(self, request, datum=None):
        return {"project_id": getattr(datum, "project_id", None)}

    def get_link_url(self, replica):
        return reverse(self.url, args=(replica.id,))


class ResetReplicaStatus(tables.LinkAction):
    name = "reset_replica_status"
    verbose_name = _("Reset replica status")
    url = "horizon:admin:shares:reset_replica_status"
    classes = ("ajax-modal", "btn-create")
    policy_rules = (("share_replica", "share_replica:reset_status"),)

    def allowed(self, request, replica=None):
        return True

    def get_policy_target(self, request, datum=None):
        return {"project_id": getattr(datum, "project_id", None)}

    def get_link_url(self, replica):
        return reverse(self.url, args=(replica.id,))


class DeleteReplica(tables.DeleteAction):
    policy_rules = (("share_replica", "share_replica:delete"),)

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Delete Replica",
            u"Delete Replicas",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Deleted Replica",
            u"Deleted Replicas",
            count
        )

    def get_policy_target(self, request, datum=None):
        return {"project_id": getattr(datum, "project_id", None)}

    def delete(self, request, obj_id):
        try:
            manila.share_replica_delete(request, obj_id)
        except Exception:
            msg = _('Unable to delete replica "%s".') % obj_id
            messages.error(request, msg)

    def allowed(self, request, replica=None):
        if replica:
            share = manila.share_get(request, replica.share_id)
            replicas = manila.share_replica_list(request, replica.share_id)
            if share.replication_type is None:
                return False
            elif (share.replication_type == 'writable' and
                  replica.status in DELETABLE_STATUSES and
                  len(replicas) > 1) or (
                      share.replication_type in ('dr', 'readable') and
                      replica.status in DELETABLE_STATUSES and
                      replica.replica_state != 'active'):
                return True
        return False

    def single(self, data_table, request, object_id):
        try:
            manila.share_replica_delete(request, object_id)
            messages.success(
                request, _('Share replica %s has been deleted.') % object_id)
        except Exception:
            msg = _('Unable to delete replica "%s".') % object_id
            messages.error(request, msg)


_DETAIL_URL = "horizon:admin:shares:replica_detail"


class ReplicasTable(replica_tables.ReplicasTable):

    class Meta(object):
        name = "replicas"
        verbose_name = _("Replicas")
        status_columns = ("status",)
        row_class = UpdateReplicaRow
        row_actions = (
            ResyncReplica,
            ResetReplicaState,
            ResetReplicaStatus,
            DeleteReplica)



# Copyright (c) 2020 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy
from horizon import exceptions
from horizon import tables
from horizon.utils.filters import parse_isotime

from manila_ui.api import manila


def get_date(message):
    return parse_isotime(message.created_at)


class DeleteUserMessage(tables.DeleteAction):
    policy_rules = (("share", "share:delete_message"),)

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Delete User Message",
            u"Delete User Messages",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Deleted User Message",
            u"Deleted User Messages",
            count
        )

    def get_policy_target(self, request, datum=None):
        project_id = None
        if datum:
            project_id = getattr(datum, "project_id", None)
        return {"project_id": project_id}

    def delete(self, request, obj_id):
        obj = self.table.get_object_by_id(obj_id)
        message_id = self.table.get_object_display(obj)
        try:
            manila.messages_delete(request, message_id)
        except Exception:
            msg = _('Unable to delete message "%s"')
            exceptions.handle(self.request, msg % message_id)
            raise


class UserMessageIDColumn(tables.Column):

    def get_link_url(self, message):
        return reverse(self.link, args=(message.message_id,))


class UserMessagesTable(tables.DataTable):
    message_id = tables.Column(
        "id",
        verbose_name=_("ID"),
        link="horizon:project:user_messages:user_messages_detail")
    message_level = tables.Column(
        "message_level",
        verbose_name=_("Message Level")
    )
    resource_type = tables.Column(
        "resource_type",
        verbose_name=_("Resource Type"))
    resource_id = tables.Column(
        "resource_id",
        verbose_name=_("Resource ID"))
    user_message = tables.Column(
        "user_message",
        verbose_name=_("User Message"))
    created_at = tables.Column(
        get_date,
        verbose_name=_("Created At"))

    def get_object_display(self, obj):
        return obj.id

    class Meta(object):
        name = "user_messages"
        verbose_name = _("User Messages")
        table_actions = (
            tables.NameFilterAction,
            DeleteUserMessage,
        )
        row_actions = (
            DeleteUserMessage,
        )



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import workflows

from openstack_dashboard.api import base

from manila_ui.api import manila as api_manila


class UpdateDefaultShareQuotasAction(workflows.Action):
    shares = forms.IntegerField(min_value=-1, label=_("Shares"))
    share_gigabytes = forms.IntegerField(
        min_value=-1, label=_("Share gigabytes"))
    share_snapshots = forms.IntegerField(
        min_value=-1, label=_("Share snapshots"))
    share_snapshot_gigabytes = forms.IntegerField(
        min_value=-1, label=_("Share snapshot gigabytes"))
    share_networks = forms.IntegerField(
        min_value=-1, label=_("Share Networks"))
    # Share group quotas are missing here because default quota update
    # for share groups is not possible yet, see LP #1871252

    def __init__(self, request, context, *args, **kwargs):
        super(UpdateDefaultShareQuotasAction, self).__init__(
            request, context, *args, **kwargs)
        disabled_quotas = context['disabled_quotas']
        for field in disabled_quotas:
            if field in self.fields:
                self.fields[field].required = False
                self.fields[field].widget = forms.HiddenInput()

    def handle(self, request, data):
        try:
            if base.is_service_enabled(request, 'share'):
                manila_data = {}
                # Share group quotas are removed here because default
                # quota update for share groups is not possible yet, see
                # LP #1871252
                allowed_updates = (
                    api_manila.MANILA_QUOTA_FIELDS -
                    {'share_groups', 'share_group_snapshots'}
                )
                for key in allowed_updates:
                    manila_data[key] = data[key]
                api_manila.default_quota_update(request, **manila_data)
                return True
        except Exception:
            exceptions.handle(request,
                              _('Unable to update default quotas.'))
            return False

    class Meta(object):
        name = _("Share")
        slug = 'update_default_share_quotas'
        help_text = _("From here you can update the default share quotas "
                      "(max limits).")


class UpdateDefaultShareQuotasStep(workflows.Step):
    action_class = UpdateDefaultShareQuotasAction
    contributes = api_manila.MANILA_QUOTA_FIELDS
    depends_on = ('disabled_quotas',)

    def prepare_action_context(self, request, context):
        try:
            quota_defaults = api_manila.default_quota_get(
                request, request.user.tenant_id)
            for field in api_manila.MANILA_QUOTA_FIELDS:
                # Resolve mismatch UI field names and data field names.
                data_field = api_manila.MANILA_QUOTA_FIELDS_DATA_MAP[field]
                context[field] = quota_defaults.get(data_field).limit
        except Exception:
            exceptions.handle(request,
                              _('Unable to retrieve default share quotas.'))
        return context

    def allowed(self, request):
        return base.is_service_enabled(request, 'share')



# Copyright (c) 2014 NetApp, Inc.
# Copyright (c) 2015 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from horizon import exceptions as horizon_exceptions
from openstack_dashboard.api import keystone as api_keystone
from unittest import mock

from manila_ui.api import manila as api_manila
from manila_ui.dashboards.admin import utils
from manila_ui.tests.dashboards.project import test_data
from manila_ui.tests import helpers as test
from manila_ui.tests.test_data import keystone_data

INDEX_URL = reverse('horizon:admin:security_services:index')


class SecurityServicesTests(test.BaseAdminViewTests):

    def setUp(self):
        super(self.__class__, self).setUp()
        self.mock_object(
            api_keystone, "tenant_list",
            mock.Mock(return_value=(keystone_data.projects, None)))
        # Reset taken list of projects to avoid test interference
        utils.PROJECTS = {}

    def test_detail_view(self):
        sec_service = test_data.sec_service
        self.mock_object(
            api_manila, "security_service_get",
            mock.Mock(return_value=sec_service))
        url = reverse(
            'horizon:admin:security_services:security_service_detail',
            args=[sec_service.id])

        res = self.client.get(url)

        self.assertContains(res, "<h1>Security Service Details: %s</h1>"
                                 % sec_service.name,
                            1, 200)
        self.assertContains(res, "<dd>%s</dd>" % sec_service.name, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % sec_service.id, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % sec_service.user, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % sec_service.server, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % sec_service.dns_ip, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % sec_service.ou, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % sec_service.domain, 1, 200)
        self.assertNoMessages()
        api_manila.security_service_get.assert_called_once_with(
            mock.ANY, sec_service.id)

    def test_detail_view_with_exception(self):
        url = reverse(
            'horizon:admin:security_services:security_service_detail',
            args=[test_data.sec_service.id])
        self.mock_object(
            api_manila, "security_service_get",
            mock.Mock(side_effect=horizon_exceptions.NotFound(404)))

        res = self.client.get(url)

        self.assertRedirectsNoFollow(res, INDEX_URL)
        api_manila.security_service_get.assert_called_once_with(
            mock.ANY, test_data.sec_service.id)

    def test_delete_security_service(self):
        security_service = test_data.sec_service
        formData = {
            'action': 'security_services__delete__%s' % security_service.id,
        }
        self.mock_object(api_manila, "security_service_delete")
        self.mock_object(
            api_manila, "security_service_list",
            mock.Mock(return_value=[test_data.sec_service]))

        res = self.client.post(INDEX_URL, formData)

        api_keystone.tenant_list.assert_called_once_with(mock.ANY)
        api_manila.security_service_delete.assert_called_once_with(
            mock.ANY, test_data.sec_service.id)
        api_manila.security_service_list.assert_called_once_with(
            mock.ANY, search_opts={'all_tenants': True})
        self.assertRedirectsNoFollow(res, INDEX_URL)



# Copyright 2017 Mirantis, Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.forms import ValidationError
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon.utils import memoized

from manila_ui.api import manila
from manila_ui.dashboards import utils


class CreateShareGroupForm(forms.SelfHandlingForm):
    name = forms.CharField(label=_("Name"), max_length="255", required=True)
    description = forms.CharField(
        label=_("Description"),
        max_length="255",
        widget=forms.Textarea(attrs={"rows": 3}),
        required=False)

    def __init__(self, request, *args, **kwargs):
        super(CreateShareGroupForm, self).__init__(request, *args, **kwargs)
        self.st_field_name_prefix = "share-type-choices-"
        self.fields["source_type"] = forms.ChoiceField(
            label=_("Source Type"),
            widget=forms.Select(attrs={
                "class": "switchable",
                "data-slug": "source",
            }),
            required=False)

        self.fields["snapshot"] = forms.ChoiceField(
            label=_("Use share group snapshot as a source"),
            widget=forms.SelectWidget(attrs={
                "class": "switched",
                "data-switch-on": "source",
                "data-source-snapshot": _("Share Group Snapshot"),
            }),
            required=True)

        if ("snapshot_id" in request.GET or
                kwargs.get("data", {}).get("snapshot")):
            try:
                snapshot = self.get_share_group_snapshot(
                    request,
                    request.GET.get(
                        "snapshot_id",
                        kwargs.get("data", {}).get("snapshot")))
                self.fields["name"].initial = snapshot.name
                self.fields["snapshot"].choices = (
                    (snapshot.id, snapshot.name or snapshot.id),
                )
                try:
                    # Set the share group type from the original share group
                    orig_sg = manila.share_group_get(
                        request, snapshot.share_group_id)
                    self.fields["sgt"].initial = orig_sg.share_group_type_id
                except Exception:
                    pass
                del self.fields["source_type"]
            except Exception:
                exceptions.handle(
                    request,
                    _("Unable to load the specified share group snapshot."))
        else:
            source_type_choices = []
            try:
                snapshot_list = manila.share_group_snapshot_list(request)
                snapshots = [s for s in snapshot_list
                             if s.status == "available"]
                if snapshots:
                    source_type_choices.append(("snapshot", _("Snapshot")))
                    self.fields["snapshot"].choices = (
                        [("", _("Choose a snapshot"))] +
                        [(s.id, s.name or s.id) for s in snapshots]
                    )
                else:
                    del self.fields["snapshot"]
            except Exception:
                exceptions.handle(
                    request,
                    _("Unable to retrieve share group snapshots."))

            if source_type_choices:
                choices = ([('none', _("No source, empty share group"))] +
                           source_type_choices)
                self.fields["source_type"].choices = choices
            else:
                del self.fields["source_type"]

            self.fields["az"] = forms.ChoiceField(
                label=_("Availability Zone"),
                widget=forms.SelectWidget(attrs={
                    "class": "switched",
                    "data-switch-on": "source",
                    "data-source-none": _("Availability Zone"),
                }),
                required=False)

            availability_zones = manila.availability_zone_list(request)
            self.fields["az"].choices = (
                [("", "")] + [(az.name, az.name) for az in availability_zones])

            share_group_types = manila.share_group_type_list(request)
            self.fields["sgt"] = forms.ChoiceField(
                label=_("Share Group Type"),
                widget=forms.fields.SelectWidget(attrs={
                    "class": "switched switchable",
                    "data-switch-on": "source",
                    "data-source-none": _("Share Group Type"),
                    "data-slug": "sgt",
                }),
                required=True)
            self.fields["sgt"].choices = (
                [(utils.transform_dashed_name(sgt.id), sgt.name) for sgt in
                 share_group_types]
            )

            # NOTE(vponomaryov): create separate set of available share types
            # for each of share group types.
            share_types = manila.share_type_list(request)
            for sgt in share_group_types:
                st_choices = (
                    [(st.id, st.name)
                     for st in share_types if st.id in sgt.share_types])
                amount_of_choices = len(st_choices)
                st_field_name = (
                    self.st_field_name_prefix +
                    utils.transform_dashed_name(sgt.id)
                )
                if amount_of_choices < 2:
                    st_field = forms.ChoiceField(
                        label=_("Share Types"),
                        choices=st_choices,
                        widget=forms.fields.SelectWidget(attrs={
                            "class": "switched",
                            "data-switch-on": "sgt",
                            "data-sgt-%s" % utils.transform_dashed_name(
                                sgt.id): _("Share Types (one available)"),
                        }),
                        required=True)
                else:
                    height = min(30 * amount_of_choices, 155)
                    st_field = forms.MultipleChoiceField(
                        label=_("Share Types"),
                        choices=st_choices,
                        widget=forms.fields.widgets.SelectMultiple(attrs={
                            "style": "max-height: %spx;" % height,
                            "class": "switched",
                            "data-switch-on": "sgt",
                            "data-sgt-%s" % utils.transform_dashed_name(
                                sgt.id): _("Share Types (multiple available)"),
                        }),
                        required=False)
                    st_field.initial = st_choices[0]
                self.fields[st_field_name] = st_field

            self.fields["share_network"] = forms.ChoiceField(
                label=_("Share Network"),
                widget=forms.fields.SelectWidget(attrs={
                    "class": "switched",
                    "data-switch-on": "source",
                    "data-source-none": _("Share Network"),
                }),
                required=False)
            share_networks = manila.share_network_list(request)
            self.fields["share_network"].choices = (
                [(sn.id, sn.name or sn.id) for sn in share_networks])

    def clean(self):
        cleaned_data = super(CreateShareGroupForm, self).clean()
        form_errors = list(self.errors)

        for error in form_errors:
            sgt_name = error.split(self.st_field_name_prefix)[-1]
            chosen_sgt = cleaned_data.get("sgt")
            if (error.startswith(self.st_field_name_prefix) and
                    sgt_name != chosen_sgt):
                cleaned_data[error] = "Not set"
                self.errors.pop(error, None)

        source_type = cleaned_data.get("source_type")
        if source_type != "snapshot":
            self.errors.pop("snapshot", None)
            share_group_type = cleaned_data.get("sgt")
            if share_group_type:
                cleaned_data["share_types"] = cleaned_data.get(
                    self.st_field_name_prefix + share_group_type)
                if isinstance(cleaned_data["share_types"], str):
                    cleaned_data["share_types"] = [cleaned_data["share_types"]]
        else:
            self.errors.pop("sgt", None)

        return cleaned_data

    def handle(self, request, data):
        try:
            source_type = data.get('source_type')
            if (data.get("snapshot") and source_type in (None, 'snapshot')):
                snapshot = self.get_share_group_snapshot(
                    request, data["snapshot"])
                snapshot_id = snapshot.id
                source_sg = manila.share_group_get(
                    request, snapshot.share_group_id)
                data['sgt'] = source_sg.share_group_type_id
            else:
                snapshot_id = None

            share_group = manila.share_group_create(
                request,
                name=data['name'],
                description=data['description'],
                share_group_type=utils.transform_dashed_name(data['sgt']),
                share_types=None if snapshot_id else data.get('share_types'),
                share_network=(
                    None if snapshot_id else data.get('share_network')),
                source_share_group_snapshot=snapshot_id,
                availability_zone=None if snapshot_id else data['az'])

            message = _('Creating share group "%s"') % data['name']
            messages.success(request, message)
            return share_group
        except ValidationError as e:
            self.api_error(e.messages[0])
            return False
        except Exception:
            exceptions.handle(request, ignore=True)
            self.api_error(_("Unable to create share group."))
            return False

    @memoized.memoized
    def get_share_group_snapshot(self, request, sg_snapshot_id):
        return manila.share_group_snapshot_get(request, sg_snapshot_id)


class UpdateShareGroupForm(forms.SelfHandlingForm):
    name = forms.CharField(
        max_length="255", label=_("Share Group Name"))
    description = forms.CharField(
        widget=forms.Textarea, label=_("Description"), required=False)

    def handle(self, request, data):
        sg_id = self.initial['share_group_id']
        try:
            manila.share_group_update(
                request, sg_id, data['name'], data['description'])
            message = _('Updating share group "%s"') % data['name']
            messages.success(request, message)
            return True
        except Exception:
            redirect = reverse("horizon:project:share_groups:index")
            exceptions.handle(
                request, _('Unable to update share group.'), redirect=redirect)
            return False



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from openstack_dashboard.test.integration_tests import helpers


class TestManilaDashboardInstalled(helpers.TestCase):
    def test_shares_page_opened(self):
        shares_page = self.home_pg.go_to_project_share_sharespage()
        self.assertEqual(shares_page.page_title,
                         'Shares - OpenStack Dashboard')

    def test_share_snapshots_page_opened(self):
        s_snaps_page = self.home_pg.go_to_project_share_sharesnapshotspage()
        self.assertEqual(s_snaps_page.page_title,
                         'Share Snapshots - OpenStack Dashboard')

    def test_share_networks_page_opened(self):
        s_networks_page = self.home_pg.go_to_project_share_sharenetworkspage()
        self.assertEqual(s_networks_page.page_title,
                         'Share Networks - OpenStack Dashboard')

    def test_security_services_page_opened(self):
        sec_serv_page = self.home_pg.go_to_project_share_securityservicespage()
        self.assertEqual(sec_serv_page.page_title,
                         'Security Services - OpenStack Dashboard')

    def test_share_groups_page_opened(self):
        share_groups_page = self.home_pg.go_to_project_share_sharegroupspage()
        self.assertEqual(share_groups_page.page_title,
                         'Share Groups - OpenStack Dashboard')

    def test_share_group_snapthots_page_opened(self):
        sg_s_page = self.home_pg.go_to_project_share_sharegroupsnapshotspage()
        self.assertEqual(sg_s_page.page_title,
                         'Share Group Snapshots - OpenStack Dashboard')


class TestManilaAdminDashboardInstalled(helpers.AdminTestCase):
    def test_shares_page_opened(self):
        shares_page = self.home_pg.go_to_admin_share_sharespage()
        self.assertEqual(shares_page.page_title,
                         'Shares - OpenStack Dashboard')

    def test_share_snapshots_page_opened(self):
        s_snapshots_page = self.home_pg.go_to_admin_share_sharesnapshotspage()
        # TODO(e0ne): fix page title and test
        self.assertEqual(s_snapshots_page.page_title,
                         'Shares - OpenStack Dashboard')

    def test_share_snapshot_types_page_opened(self):
        share_types_page = (
            self.home_pg.go_to_admin_share_sharetypespage())
        self.assertEqual(share_types_page.page_title,
                         'Share Types - OpenStack Dashboard')

    def test_share_networks_page_opened(self):
        s_networks_page = self.home_pg.go_to_admin_share_sharenetworkspage()
        self.assertEqual(s_networks_page.page_title,
                         'Share Networks - OpenStack Dashboard')

    def test_security_services_page_opened(self):
        sec_serv_page = self.home_pg.go_to_admin_share_securityservicespage()
        self.assertEqual(sec_serv_page.page_title,
                         'Security Services - OpenStack Dashboard')

    def test_share_servers_page_opened(self):
        share_serv_page = self.home_pg.go_to_admin_share_shareserverspage()
        self.assertEqual(share_serv_page.page_title,
                         'Share Servers - OpenStack Dashboard')

    def test_share_instences_page_opened(self):
        sec_serv_page = self.home_pg.go_to_admin_share_shareinstancespage()
        self.assertEqual(sec_serv_page.page_title,
                         'Share Instances - OpenStack Dashboard')

    def test_share_groups_page_opened(self):
        share_groups_page = self.home_pg.go_to_admin_share_sharegroupspage()
        self.assertEqual(share_groups_page.page_title,
                         'Share Groups - OpenStack Dashboard')

    def test_share_group_snapthots_page_opened(self):
        sg_s_page = self.home_pg.go_to_admin_share_sharegroupsnapshotspage()
        self.assertEqual(sg_s_page.page_title,
                         'Share Group Snapshots - OpenStack Dashboard')

    def test_share_group_types_page_opened(self):
        sg_types_page = self.home_pg.go_to_admin_share_sharegrouptypespage()
        self.assertEqual(sg_types_page.page_title,
                         'Share Group Types - OpenStack Dashboard')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.urls import re_path

from manila_ui.dashboards.admin.shares.replicas import views as replica_views
from manila_ui.dashboards.admin.shares import views
from manila_ui import features


urlpatterns = [
    re_path(
        r'^$',
        views.SharesView.as_view(),
        name='index'),
    re_path(
        r'^(?P<share_id>[^/]+)/$',
        views.DetailView.as_view(),
        name='detail'),
    re_path(
        r'^manage$',
        views.ManageShareView.as_view(),
        name='manage'),
    re_path(
        r'^unmanage/(?P<share_id>[^/]+)$',
        views.UnmanageShareView.as_view(),
        name='unmanage'),
]

if features.is_replication_enabled():
    urlpatterns.extend([
        re_path(
            r'^(?P<share_id>[^/]+)/replicas/$',
            replica_views.ManageReplicasView.as_view(),
            name='manage_replicas'),
        re_path(
            r'^replica/(?P<replica_id>[^/]+)$',
            replica_views.DetailReplicaView.as_view(),
            name='replica_detail'),
        re_path(
            r'^replica/(?P<replica_id>[^/]+)/resync_replica$',
            replica_views.ResyncReplicaView.as_view(),
            name='resync_replica'),
        re_path(
            r'^replica/(?P<replica_id>[^/]+)/reset_replica_status$',
            replica_views.ResetReplicaStatusView.as_view(),
            name='reset_replica_status'),
        re_path(
            r'^replica/(?P<replica_id>[^/]+)/reset_replica_state$',
            replica_views.ResetReplicaStateView.as_view(),
            name='reset_replica_state'),
    ])

if features.is_migration_enabled():
    urlpatterns.extend([
        re_path(
            r'^migration_start/(?P<share_id>[^/]+)$',
            views.MigrationStartView.as_view(),
            name='migration_start'),
        re_path(
            r'^migration_complete/(?P<share_id>[^/]+)$',
            views.MigrationCompleteView.as_view(),
            name='migration_complete'),
        re_path(
            r'^migration_cancel/(?P<share_id>[^/]+)$',
            views.MigrationCancelView.as_view(),
            name='migration_cancel'),
        re_path(
            r'^migration_get_progress/(?P<share_id>[^/]+)$',
            views.MigrationGetProgressView.as_view(),
            name='migration_get_progress'),
    ])



# Copyright 2016 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from django.conf import settings


settings.POLICY_FILES.update({
    'share': 'manila_policy.yaml',
})

settings.DEFAULT_POLICY_FILES.update({
    'share': 'default_policies/manila.yaml'
})

# Sample
# settings.LOGGING['loggers'].update({
#     'manilaclient': {
#         'handlers': ['console'],
#         'level': 'DEBUG',
#         'propagate': False,
#     }
# })

# The OPENSTACK_MANILA_FEATURES settings can be used to enable or disable
# the UI for the various services provided by Manila.
OPENSTACK_MANILA_FEATURES = {
    'enable_share_groups': True,
    'enable_replication': True,
    'enable_migration': True,
    'enable_public_share_type_creation': True,
    'enable_public_share_group_type_creation': True,
    'enable_public_shares': True,
    'enabled_share_protocols': ['NFS', 'CIFS', 'GlusterFS', 'HDFS', 'CephFS',
                                'MapRFS'],
}



# Copyright (c) 2020 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon.utils import memoized

from manila_ui.api import manila
from manila_ui.dashboards.admin.user_messages import tables as admin_tables
from manila_ui.dashboards.admin.user_messages import tabs as admin_tabs
from manila_ui.dashboards.admin import utils
from manila_ui.dashboards.project.user_messages import views as project_views


class UserMessagesAdminView(project_views.UserMessagesView):
    table_classes = (
        admin_tables.UserMessagesAdminTable,
    )
    template_name = "admin/user_messages/index.html"

    @memoized.memoized_method
    def get_user_messages_data(self):
        try:
            messages = manila.messages_list(self.request)
            utils.set_project_name_to_objects(self.request, messages)
        except Exception:
            msg = _("Unable to retrieve messages list.")
            exceptions.handle(self.request, msg)
            return []
        return messages


class UserMessagesAdminDetailView(project_views.UserMessagesDetailView):
    tab_group_class = admin_tabs.UserMessagesDetailTabs
    template_name = 'admin/user_messages/detail.html'
    redirect_url = reverse_lazy('horizon:admin:user_messages:index')



# Copyright 2017 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Admin views for managing share networks.
"""

from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import tables
from horizon.utils import memoized
from openstack_dashboard.api import base
from openstack_dashboard.api import neutron

from manila_ui.api import manila
from manila_ui.dashboards.admin.share_networks import tables as sn_tables
from manila_ui.dashboards.admin.share_networks import tabs as sn_tabs
from manila_ui.dashboards.admin import utils
from manila_ui.dashboards.project.share_networks import views as p_views


class ShareNetworksView(tables.MultiTableView):
    table_classes = (
        sn_tables.ShareNetworksTable,
    )
    template_name = "admin/share_networks/index.html"
    page_title = _("Share Networks")

    @memoized.memoized_method
    def get_share_networks_data(self):
        try:
            share_networks = manila.share_network_list(
                self.request, detailed=True, search_opts={'all_tenants': True})
            if base.is_service_enabled(self.request, 'network'):
                neutron_net_names = dict(
                    (net.id, net.name)
                    for net in neutron.network_list(self.request))
                neutron_subnet_names = dict(
                    (net.id, net.name)
                    for net in neutron.subnet_list(self.request))
                for sn in share_networks:
                    sn.neutron_net = neutron_net_names.get(
                        sn.neutron_net_id) or sn.neutron_net_id or "-"
                    sn.neutron_subnet = neutron_subnet_names.get(
                        sn.neutron_subnet_id) or sn.neutron_subnet_id or "-"
        except Exception:
            share_networks = []
            exceptions.handle(
                self.request, _("Unable to retrieve share networks"))
        utils.set_project_name_to_objects(self.request, share_networks)
        return share_networks


class ShareNetworkDetailView(p_views.Detail):
    tab_group_class = sn_tabs.ShareNetworkDetailTabs
    template_name = "admin/share_networks/detail.html"
    redirect_url = reverse_lazy("horizon:admin:share_networks:index")



# Copyright (c) 2014 NetApp, Inc.
# Copyright (c) 2015 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from horizon import exceptions as horizon_exceptions
from neutronclient.client import exceptions
from openstack_dashboard.api import keystone as api_keystone
from openstack_dashboard.api import neutron as api_neutron
from unittest import mock

from manila_ui.api import manila as api_manila
from manila_ui.dashboards.admin import utils
from manila_ui.tests.dashboards.project import test_data
from manila_ui.tests import helpers as test
from manila_ui.tests.test_data import keystone_data

INDEX_URL = reverse('horizon:admin:share_networks:index')


class ShareNetworksTests(test.BaseAdminViewTests):

    def setUp(self):
        super(self.__class__, self).setUp()
        self.mock_object(
            api_keystone, "tenant_list",
            mock.Mock(return_value=(keystone_data.projects, None)))
        # Reset taken list of projects to avoid test interference
        utils.PROJECTS = {}

    def test_detail_view(self):
        share_net = test_data.active_share_network
        sec_service = test_data.sec_service
        self.mock_object(
            api_manila, "share_server_list", mock.Mock(return_value=[]))
        self.mock_object(
            api_manila, "share_network_get", mock.Mock(return_value=share_net))
        self.mock_object(
            api_manila, "share_network_security_service_list",
            mock.Mock(return_value=[sec_service]))
        network = self.networks.first()
        subnet = self.subnets.first()
        self.mock_object(
            api_neutron, "network_get", mock.Mock(return_value=network))
        self.mock_object(
            api_neutron, "subnet_get", mock.Mock(return_value=subnet))
        url = reverse('horizon:admin:share_networks:share_network_detail',
                      args=[share_net.id])

        res = self.client.get(url)

        self.assertContains(res, "<h1>Share Network Details: %s</h1>"
                                 % share_net.name,
                            1, 200)
        self.assertContains(res, "<dd>%s</dd>" % share_net.name, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % share_net.id, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % network.name_or_id, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % subnet.name_or_id, 1, 200)
        self.assertContains(res, "<a href=\"/admin/security_services"
                                 "/%s\">%s</a>" % (sec_service.id,
                                                   sec_service.name), 1, 200)
        self.assertNoMessages()
        api_manila.share_network_security_service_list.assert_called_once_with(
            mock.ANY, share_net.id)
        api_manila.share_server_list.assert_called_once_with(
            mock.ANY, search_opts={'share_network_id': share_net.id})
        api_manila.share_network_get.assert_called_once_with(
            mock.ANY, share_net.id)
        api_neutron.network_get.assert_called_once_with(
            mock.ANY, share_net.neutron_net_id)
        api_neutron.subnet_get.assert_called_once_with(
            mock.ANY, share_net.neutron_subnet_id)

    def test_detail_view_network_not_found(self):
        share_net = test_data.active_share_network
        sec_service = test_data.sec_service
        url = reverse('horizon:admin:share_networks:share_network_detail',
                      args=[share_net.id])
        self.mock_object(
            api_manila, "share_server_list", mock.Mock(return_value=[]))
        self.mock_object(
            api_manila, "share_network_get", mock.Mock(return_value=share_net))
        self.mock_object(
            api_manila, "share_network_security_service_list",
            mock.Mock(return_value=[sec_service]))
        self.mock_object(
            api_neutron, "network_get", mock.Mock(
                side_effect=exceptions.NeutronClientException('fake', 500)))
        self.mock_object(
            api_neutron, "subnet_get", mock.Mock(
                side_effect=exceptions.NeutronClientException('fake', 500)))

        res = self.client.get(url)

        self.assertContains(res, "<h1>Share Network Details: %s</h1>"
                                 % share_net.name,
                            1, 200)
        self.assertContains(res, "<dd>%s</dd>" % share_net.name, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % share_net.id, 1, 200)
        self.assertContains(res, "<dd>Unknown</dd>", 2, 200)
        self.assertNotContains(res, "<dd>%s</dd>" % share_net.neutron_net_id)
        self.assertNotContains(res,
                               "<dd>%s</dd>" % share_net.neutron_subnet_id)
        self.assertContains(res, "<a href=\"/admin/security_services"
                                 "/%s\">%s</a>" % (sec_service.id,
                                                   sec_service.name), 1, 200)
        self.assertNoMessages()
        api_manila.share_network_security_service_list.assert_called_once_with(
            mock.ANY, share_net.id)
        api_manila.share_server_list.assert_called_once_with(
            mock.ANY, search_opts={'share_network_id': share_net.id})
        api_manila.share_network_get.assert_called_once_with(
            mock.ANY, share_net.id)
        api_neutron.network_get.assert_called_once_with(
            mock.ANY, share_net.neutron_net_id)
        api_neutron.subnet_get.assert_called_once_with(
            mock.ANY, share_net.neutron_subnet_id)

    def test_detail_view_with_exception(self):
        url = reverse('horizon:admin:share_networks:share_network_detail',
                      args=[test_data.active_share_network.id])
        self.mock_object(
            api_manila, "share_network_get",
            mock.Mock(side_effect=horizon_exceptions.NotFound(404)))

        res = self.client.get(url)

        self.assertRedirectsNoFollow(res, INDEX_URL)
        api_manila.share_network_get.assert_called_once_with(
            mock.ANY, test_data.active_share_network.id)

    def test_delete_share_network(self):
        share_network = test_data.inactive_share_network
        formData = {'action': 'share_networks__delete__%s' % share_network.id}
        self.mock_object(
            api_neutron, "network_list", mock.Mock(return_value=[]))
        self.mock_object(
            api_neutron, "subnet_list", mock.Mock(return_value=[]))
        self.mock_object(api_manila, "share_network_delete")
        self.mock_object(
            api_manila, "share_network_list",
            mock.Mock(return_value=[
                test_data.active_share_network,
                test_data.inactive_share_network]))

        res = self.client.post(INDEX_URL, formData)

        api_keystone.tenant_list.assert_called_once_with(mock.ANY)
        api_manila.share_network_delete.assert_called_once_with(
            mock.ANY, test_data.inactive_share_network.id)
        api_manila.share_network_list.assert_called_once_with(
            mock.ANY, detailed=True, search_opts={'all_tenants': True})
        api_neutron.network_list.assert_called_once_with(mock.ANY)
        api_neutron.subnet_list.assert_called_once_with(mock.ANY)
        self.assertRedirectsNoFollow(res, INDEX_URL)



# Copyright (c) 2014 NetApp, Inc.
# Copyright (c) 2015 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from openstack_dashboard.api import keystone as api_keystone
from openstack_dashboard.api import neutron as api_neutron
from unittest import mock

from manila_ui.api import manila as api_manila
from manila_ui.dashboards.admin import utils
from manila_ui.tests.dashboards.project import test_data
from manila_ui.tests import helpers as test
from manila_ui.tests.test_data import keystone_data

INDEX_URL = reverse('horizon:admin:share_types:index')


class ShareTypeTests(test.BaseAdminViewTests):

    def setUp(self):
        super(self.__class__, self).setUp()
        self.share_type = test_data.share_type
        self.url = reverse('horizon:admin:share_types:update_type',
                           args=[self.share_type.id])
        self.mock_object(
            api_manila, "share_type_get",
            mock.Mock(return_value=self.share_type))
        self.mock_object(
            api_keystone, "tenant_list",
            mock.Mock(return_value=(keystone_data.projects, None)))
        self.mock_object(
            api_neutron, "is_service_enabled", mock.Mock(return_value=[True]))
        # Reset taken list of projects to avoid test interference
        utils.PROJECTS = {}

    def test_create_share_type(self):
        url = reverse('horizon:admin:share_types:create_type')
        data = {
            'is_public': True,
            'name': 'my_share_type',
            'description': 'share_type_description',
            'spec_driver_handles_share_servers': 'False'
        }
        form_data = data.copy()
        form_data['spec_driver_handles_share_servers'] = 'false'
        self.mock_object(api_manila, "share_type_create")

        res = self.client.post(url, data)

        api_manila.share_type_create.assert_called_once_with(
            mock.ANY, form_data['name'],
            form_data['spec_driver_handles_share_servers'],
            description=form_data['description'],
            is_public=form_data['is_public'])
        self.assertRedirectsNoFollow(res, INDEX_URL)

    def test_update_share_type_get(self):
        res = self.client.get(self.url)

        api_manila.share_type_get.assert_called_once_with(
            mock.ANY, self.share_type.id)
        self.assertNoMessages()
        self.assertTemplateUsed(res, 'admin/share_types/update.html')

    def test_update_share_type_post(self):
        data = {
            'name': 'share_type_update',
            'description': 'share_type_description',
            'is_public': True,
            'extra_specs': 'fork_key=fork_val'
        }
        form_data = {
            'name': 'share_type_update',
            'description': 'share_type_description',
            'is_public': True,
            'extra_specs': {'fork_key': 'fork_val'},
        }
        self.mock_object(api_manila, "share_type_set_extra_specs")
        self.mock_object(api_manila, "share_type_update")

        res = self.client.post(self.url, data)

        api_manila.share_type_set_extra_specs.assert_called_once_with(
            mock.ANY, self.share_type.id, form_data['extra_specs'])
        api_manila.share_type_update.assert_called_once_with(
            mock.ANY, self.share_type.id, data['name'], data['description'],
            data['is_public'])
        self.assertRedirectsNoFollow(res, INDEX_URL)



# Copyright 2020 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from django.urls import reverse
from unittest import mock

from horizon import exceptions as horizon_exceptions

from manila_ui.api import manila as api_manila
from manila_ui.tests.dashboards.project import test_data
from manila_ui.tests import helpers as test

INDEX_URL = reverse('horizon:project:user_messages:index')


@ddt.ddt
class UserMessagesViewTests(test.TestCase):

    @ddt.data(None, Exception('fake'))
    def test_view(self, exc):
        message_1 = test_data.fake_message_1
        message_2 = test_data.fake_message_2
        message_3 = test_data.fake_message_3
        fake_message_list = [message_1, message_2, message_3]
        url = reverse('horizon:project:user_messages:index')
        self.mock_object(
            api_manila, "messages_list",
            mock.Mock(side_effect=exc, return_value=fake_message_list))

        self.client.get(url)

        self.assertNoMessages()
        api_manila.messages_list.assert_called_once_with(mock.ANY)

    @ddt.data(None, Exception('fake'))
    def test_delete_message(self, exc):
        message = test_data.fake_message_1
        formData = {'action': 'user_messages__delete__%s' % message.id}
        self.mock_object(api_manila, "messages_delete",
                         mock.Mock(side_effect=exc))
        self.mock_object(
            api_manila, "messages_list",
            mock.Mock(return_value=[message]))

        res = self.client.post(INDEX_URL, formData)

        self.assertEqual(res.status_code, 302)

        self.assertRedirectsNoFollow(res, INDEX_URL)
        api_manila.messages_list.assert_called_once_with(mock.ANY)
        api_manila.messages_delete.assert_called_once_with(
            mock.ANY, test_data.fake_message_1.id)


@ddt.ddt
class UserMessagesDetailViewTests(test.TestCase):

    def test_detail_view(self):
        message = test_data.fake_message_1
        url = reverse('horizon:project:user_messages:user_messages_detail',
                      args=[message.id])
        self.mock_object(
            api_manila, "messages_get", mock.Mock(return_value=message))

        res = self.client.get(url)

        self.assertContains(res, "<h1>User Message Details: %s</h1>"
                                 % message.id,
                            1, 200)
        self.assertContains(res, "<dd>%s</dd>" % message.id, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % message.action_id, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % message.user_message, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % message.message_level, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % message.resource_type, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % message.resource_id, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % message.request_id, 1, 200)

        self.assertNoMessages()
        api_manila.messages_get.assert_called_once_with(
            mock.ANY, message.id)

    def test_detail_view_with_exception(self):
        message = test_data.fake_message_1
        url = reverse(
            'horizon:project:user_messages:user_messages_detail',
            args=[message.id])
        self.mock_object(
            api_manila, "messages_get",
            mock.Mock(side_effect=horizon_exceptions.NotFound(404)))
        res = self.client.get(url)
        self.assertRedirectsNoFollow(res, INDEX_URL)
        api_manila.messages_get.assert_called_once_with(mock.ANY, message.id)



# Copyright (c) 2014 NetApp, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.conf import settings
from django.forms import ValidationError
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import messages
from oslo_utils import strutils

from manila_ui.api import manila
from manila_ui.dashboards import utils


def _get_id_if_name_empty(data):
    result = data.get('name', None)
    if not result:
        result = data.get('id')
    if not result:
        result = ''
    return result


class MigrationStart(forms.SelfHandlingForm):
    name = forms.CharField(
        label=_("Share Name"),
        widget=forms.TextInput(attrs={'readonly': 'readonly'}))
    share_id = forms.CharField(
        label=_("ID"),
        widget=forms.TextInput(attrs={'readonly': 'readonly'}))
    host = forms.ChoiceField(
        label=_("Host to migrate share"),
        help_text=_("Destination host and pool where share will be migrated "
                    "to."))
    force_host_assisted_migration = forms.BooleanField(
        label=_("Force Host Assisted Migration"),
        required=False, initial=False,
        help_text=_("Enforces the use of the host-assisted migration approach,"
                    " which bypasses driver optimizations."))
    nondisruptive = forms.BooleanField(
        label=_("Nondisruptive"),
        required=False, initial=True,
        help_text=_("Enforces migration to be nondisruptive. If set to True, "
                    "host-assisted migration will not be attempted."))
    writable = forms.BooleanField(
        label=_("Writable"), required=False, initial=True,
        help_text=_("Enforces migration to keep the share writable while "
                    "contents are being moved. If set to True, host-assisted "
                    "migration will not be attempted."))
    preserve_metadata = forms.BooleanField(
        label=_("Preserve Metadata"), required=False, initial=True,
        help_text=_("Enforces migration to preserve all file metadata when "
                    "moving its contents. If set to True, host-assisted "
                    "migration will not be attempted."))
    preserve_snapshots = forms.BooleanField(
        label=_("Preserve Snapshots"), required=False, initial=True,
        help_text=_("Enforces migration of the share snapshots to the "
                    "destination. If set to True, host-assisted migration will"
                    " not be attempted."))
    new_share_network = forms.ChoiceField(
        label=_("New share network to be set in migrated share"),
        required=False,
        help_text=_('Specify the new share network for the share. Do not '
                    'specify this parameter if the migrating share has to be '
                    'retained within its current share network.'))
    new_share_type = forms.ChoiceField(
        label=_("New share type to be set in migrating share"), required=False,
        help_text=_('Specify the new share type for the share. Do not specify '
                    'this parameter if the migrating share has to be retained '
                    'with its current share type.'))

    def __init__(self, request, *args, **kwargs):
        super(MigrationStart, self).__init__(request, *args, **kwargs)
        share_networks = manila.share_network_list(request)
        share_types = manila.share_type_list(request)
        dests = manila.pool_list(request)
        dest_choices = [('', '')] + [(d.name, d.name) for d in dests]
        st_choices = [('', '')] + [(st.id, st.name) for st in share_types]
        sn_choices = (
            [('', '')] +
            [(sn.id, sn.name or sn.id) for sn in share_networks])
        self.fields['host'].choices = dest_choices
        self.fields['new_share_type'].choices = st_choices
        self.fields['new_share_network'].choices = sn_choices

    def handle(self, request, data):
        share_name = _get_id_if_name_empty(data)
        try:
            manila.migration_start(
                request, self.initial['share_id'],
                force_host_assisted_migration=(
                    data['force_host_assisted_migration']),
                writable=data['writable'],
                preserve_metadata=data['preserve_metadata'],
                preserve_snapshots=data['preserve_snapshots'],
                nondisruptive=data['nondisruptive'],
                dest_host=data['host'],
                new_share_network_id=data['new_share_network'],
                new_share_type_id=data['new_share_type'])

            messages.success(
                request,
                _('Successfully sent the request to migrate share: %s.')
                % share_name)
            return True
        except Exception:
            redirect = reverse("horizon:admin:shares:index")
            exceptions.handle(
                request, _("Unable to migrate share %s.") % share_name,
                redirect=redirect)
        return False


class MigrationForms(forms.SelfHandlingForm):
    name = forms.CharField(
        label=_("Share Name"), required=False,
        widget=forms.TextInput(attrs={'readonly': 'readonly'}))
    share_id = forms.CharField(
        label=_("ID"), required=False,
        widget=forms.TextInput(attrs={'readonly': 'readonly'}))


class MigrationComplete(MigrationForms):

    def handle(self, request, data):
        share_name = _get_id_if_name_empty(data)
        try:
            manila.migration_complete(request, self.initial['share_id'])
            messages.success(
                request,
                _('Successfully sent the request to complete migration of '
                  ' share: %s.') % share_name)
            return True
        except Exception:
            exceptions.handle(request, _("Unable to complete migration "
                                         "of share %s.") % share_name)
        return False


class MigrationGetProgress(MigrationForms):

    def handle(self, request, data):
        share_name = _get_id_if_name_empty(data)
        try:
            result = manila.migration_get_progress(request,
                                                   self.initial['share_id'])
            progress = result[1]
            messages.success(
                request,
                _('Migration of share %(name)s is at %(progress)s percent.') %
                {'name': share_name, 'progress': progress['total_progress']})
            return True
        except Exception:
            exceptions.handle(request, _("Unable to obtain progress of "
                                         "migration of share %s at this "
                                         "moment.") % share_name)
        return False


class MigrationCancel(MigrationForms):

    def handle(self, request, data):
        share_name = _get_id_if_name_empty(data)
        try:
            manila.migration_cancel(request, self.initial['share_id'])
            messages.success(
                request,
                _('Successfully sent the request to cancel migration of '
                  ' share: %s.') % share_name)
            return True
        except Exception:
            exceptions.handle(request, _("Unable to cancel migration of share"
                                         " %s at this moment.") % share_name)
        return False


class ManageShare(forms.SelfHandlingForm):
    name = forms.CharField(
        max_length=255, label=_("Share Name"), required=False,
        help_text=_("Share name to be assigned"))
    description = forms.CharField(
        max_length=255, label=_("Description"), required=False,
        widget=forms.Textarea(
            attrs={'class': 'modal-body-fixed-width', 'rows': 4}))

    host = forms.CharField(
        max_length=255, label=_("Host of share"), required=True,
        help_text=_(
            "Host where share is located, example: some.host@driver[#pool]"))
    export_location = forms.CharField(
        max_length=255, label=_("Export location"), required=True,
        help_text=_("Export location of share. Example for NFS: "
                    "1.2.3.4:/path/to/share"))

    protocol = forms.ChoiceField(label=_("Share Protocol"), required=True)

    share_type = forms.ChoiceField(label=_("Share Type"), required=True)

    driver_options = forms.CharField(
        max_length=255, required=False,
        label=_("Driver options ('volume_id' for Generic driver, etc...)"),
        help_text=_("key=value pairs per line can be set"),
        widget=forms.Textarea(
            attrs={'class': 'modal-body-fixed-width', 'rows': 2}))
    is_public = forms.BooleanField(
        label=_("Public"), required=False, initial=False,
        help_text=("Defines whether this share is available for all or not."))

    def __init__(self, request, *args, **kwargs):
        super(ManageShare, self).__init__(request, *args, **kwargs)
        share_types = manila.share_type_list(request)
        # NOTE(vponomaryov): choose only those share_types that have spec
        # 'driver_handles_share_servers' set to 'False' value or alias of it.
        dhss_key = 'driver_handles_share_servers'
        st_choices = [('', ''), ]
        for st in share_types:
            dhss = st.to_dict()['extra_specs'].get(dhss_key)
            if dhss and dhss.lower() in strutils.FALSE_STRINGS:
                st_choices.append((st.name, st.name))
        self.fields['share_type'].choices = st_choices
        # NOTE(vkmc): choose only those share protocols that are enabled
        # FIXME(vkmc): this should be better implemented by having a
        # capabilities endpoint on the control plane.
        manila_features = getattr(settings, 'OPENSTACK_MANILA_FEATURES', {})
        self.enabled_share_protocols = manila_features.get(
            'enabled_share_protocols',
            ['NFS', 'CIFS', 'GlusterFS', 'HDFS', 'CephFS', 'MapRFS'])
        self.fields['protocol'].choices = ([(' ', ' ')] +
                                           [(enabled_proto, enabled_proto)
                                           for enabled_proto in
                                           self.enabled_share_protocols])

    def handle(self, request, data):
        try:
            driver_options = data.get('driver_options') or {}
            driver_options_error_msg = _(
                "Got improper value for field 'driver_options'. "
                "Expected only pairs of key=value.")
            if driver_options and isinstance(driver_options, str):
                try:
                    set_dict, unset_list = utils.parse_str_meta(driver_options)
                    if unset_list:
                        raise ValidationError(message=driver_options_error_msg)
                    driver_options = set_dict
                except ValidationError as e:
                    self.api_error(e.messages[0])
                    return False
            elif not isinstance(driver_options, dict):
                self.api_error(driver_options_error_msg)
                return False

            manila.share_manage(
                request,
                service_host=data['host'],
                protocol=data['protocol'],
                export_path=data['export_location'],
                driver_options=driver_options,
                share_type=data['share_type'],
                name=data['name'],
                description=data['description'],
                is_public=data['is_public'])

            share_name = data.get('name', data.get('id'))
            messages.success(
                request,
                _('Successfully sent the request to manage share: %s')
                % share_name)
            return True
        except Exception:
            exceptions.handle(request, _("Unable to manage share"))
        return False


class UnmanageShare(forms.SelfHandlingForm):
    name = forms.CharField(
        label=_("Share Name"), required=False,
        widget=forms.TextInput(attrs={'readonly': 'readonly'}))
    host = forms.CharField(
        label=_("Host"), required=False,
        widget=forms.TextInput(attrs={'readonly': 'readonly'}))
    share_id = forms.CharField(
        label=_("ID"), required=False,
        widget=forms.TextInput(attrs={'readonly': 'readonly'}))

    def handle(self, request, data):
        try:
            manila.share_unmanage(request, self.initial['share_id'])
            messages.success(
                request,
                _('Successfully sent the request to unmanage share: %s')
                % data['name'])
            return True
        except Exception:
            exceptions.handle(request, _("Unable to unmanage share."))
        return False



# Copyright (c) 2014 NetApp, Inc.
# Copyright (c) 2015 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from django.urls import reverse
from openstack_dashboard.api import keystone as api_keystone
from openstack_dashboard.api import neutron as api_neutron
from unittest import mock

from manila_ui.api import manila as api_manila
from manila_ui.dashboards.admin import utils
from manila_ui.tests.dashboards.project import test_data
from manila_ui.tests import helpers as test
from manila_ui.tests.test_data import keystone_data

INDEX_URL = reverse('horizon:admin:shares:index')


@ddt.ddt
class SharesTests(test.BaseAdminViewTests):

    def setUp(self):
        super(self.__class__, self).setUp()
        self.mock_object(utils.timeutils, 'now', mock.Mock(return_value=1))
        self.mock_object(
            api_keystone, "tenant_list",
            mock.Mock(return_value=(keystone_data.projects, None)))
        # Reset taken list of projects to avoid test interference
        utils.PROJECTS = {}

    def test_index(self):
        snaps = [test_data.snapshot]
        shares = [test_data.share, test_data.nameless_share,
                  test_data.other_share]
        utils.timeutils.now.side_effect = [4] + [24 + i for i in range(4)]

        self.mock_object(
            api_manila, "share_list", mock.Mock(return_value=shares))
        self.mock_object(
            api_manila, "share_snapshot_list", mock.Mock(return_value=snaps))
        self.mock_object(
            api_neutron, "is_service_enabled", mock.Mock(return_value=[True]))

        res = self.client.get(INDEX_URL)

        self.assertTemplateUsed(res, 'admin/shares/index.html')
        self.assertEqual(res.status_code, 200)
        api_manila.share_list.assert_called_with(
            mock.ANY, search_opts={'all_tenants': True})
        api_manila.share_snapshot_list.assert_called_with(
            mock.ANY, detailed=True, search_opts={'all_tenants': True})
        api_keystone.tenant_list.assert_called_once_with(mock.ANY)

    def test_delete_share(self):
        url = reverse('horizon:admin:shares:index')
        share = test_data.share
        formData = {'action': 'shares__delete__%s' % share.id}
        self.mock_object(
            api_manila, "share_snapshot_list", mock.Mock(return_value=[]))
        self.mock_object(api_manila, "share_delete")
        self.mock_object(
            api_manila, "share_get", mock.Mock(return_value=share))
        self.mock_object(
            api_manila, "share_list", mock.Mock(return_value=[share]))

        res = self.client.post(url, formData)

        api_keystone.tenant_list.assert_called_once_with(mock.ANY)
        api_manila.share_delete.assert_called_once_with(
            mock.ANY, share.id, share_group_id=share.share_group_id)
        api_manila.share_list.assert_called_once_with(
            mock.ANY, search_opts={'all_tenants': True})
        api_manila.share_snapshot_list.assert_called_once_with(
            mock.ANY, detailed=True, search_opts={'all_tenants': True})
        self.assertRedirectsNoFollow(res, INDEX_URL)

    @ddt.data(None, Exception('fake'))
    def test_migration_start_post(self, exc):
        share = test_data.share
        url = reverse('horizon:admin:shares:migration_start',
                      args=[share.id])
        sn_choices = [test.FakeEntity('sn1_id', 'sn1_name'),
                      test.FakeEntity('sn2_id', 'sn2_name')]

        st_choices = [test.FakeEntity('st1_id', 'st1_name'),
                      test.FakeEntity('st2_id', 'st2_name')]

        dest_choices = [
            test.FakeEntity('ubuntu@beta#BETA', 'ubuntu@beta#BETA'),
            test.FakeEntity('ubuntu@alpha#ALPHA', 'ubuntu@alpha#ALPHA')
        ]

        formData = {
            'share_id': share.id,
            'name': share.name,
            'host': 'ubuntu@alpha#ALPHA',
            'writable': True,
            'preserve_metadata': True,
            'preserve_snapshots': True,
            'force_host_assisted_migration': True,
            'nondisruptive': True,
            'new_share_network': 'sn2_id',
            'new_share_type': 'st2_id',
        }
        self.mock_object(
            api_manila, "share_get", mock.Mock(return_value=share))
        self.mock_object(
            api_manila, "migration_start", mock.Mock(side_effect=exc))
        self.mock_object(
            api_manila, "share_network_list",
            mock.Mock(return_value=sn_choices))
        self.mock_object(
            api_manila, "share_type_list",
            mock.Mock(return_value=st_choices))
        self.mock_object(
            api_manila, "pool_list",
            mock.Mock(return_value=dest_choices))

        res = self.client.post(url, formData)

        api_manila.share_get.assert_called_once_with(mock.ANY, share.id)
        api_manila.migration_start.assert_called_once_with(
            mock.ANY, formData['share_id'],
            dest_host=formData['host'],
            force_host_assisted_migration=(
                formData['force_host_assisted_migration']),
            writable=formData['writable'],
            preserve_metadata=formData['preserve_metadata'],
            preserve_snapshots=formData['preserve_snapshots'],
            nondisruptive=formData['nondisruptive'],
            new_share_network_id=formData['new_share_network'],
            new_share_type_id=formData['new_share_type'])
        api_manila.share_network_list.assert_called_once_with(mock.ANY)
        api_manila.share_type_list.assert_called_once_with(mock.ANY)
        api_manila.pool_list.assert_called_once_with(mock.ANY)
        self.assertEqual(302, res.status_code)
        self.assertTemplateNotUsed(res, 'admin/shares/migration_start.html')
        self.assertRedirectsNoFollow(res, INDEX_URL)

    @ddt.data('migration_start', 'migration_cancel', 'migration_complete',
              'migration_get_progress')
    def test_migration_forms_open_form_successfully(self, method):
        share = test_data.share
        url = reverse('horizon:admin:shares:' + method, args=[share.id])

        self.mock_object(
            api_manila, "share_get", mock.Mock(return_value=share))
        self.mock_object(api_manila, method)

        if method == 'migration_start':
            self.mock_object(
                api_manila, "share_network_list",
                mock.Mock(
                    return_value=[test.FakeEntity('sn1_id', 'sn1_name'),
                                  test.FakeEntity('sn2_id', 'sn2_name')]))

            self.mock_object(
                api_manila, "share_type_list",
                mock.Mock(
                    return_value=[test.FakeEntity('st1_id', 'st1_name'),
                                  test.FakeEntity('st2_id', 'st2_name')]))

            self.mock_object(
                api_manila, "pool_list",
                mock.Mock(return_value=[test.FakeEntity('ubuntu@beta#BETA',
                                                        'ubuntu@beta#BETA'),
                                        test.FakeEntity('ubuntu@alpha#ALPHA',
                                                        'ubuntu@alpha#ALPHA')
                                        ]))

        res = self.client.get(url)

        api_manila.share_get.assert_called_once_with(mock.ANY, share.id)

        self.assertFalse(getattr(api_manila, method).called)

        self.assertEqual(res.status_code, 200)
        self.assertTemplateUsed(res, 'admin/shares/' + method + '.html')

        if method == 'migration_start':
            api_manila.share_network_list.assert_called_once_with(mock.ANY)
            api_manila.share_type_list.assert_called_once_with(mock.ANY)
            api_manila.pool_list.assert_called_once_with(mock.ANY)

    @ddt.data('migration_start', 'migration_cancel', 'migration_complete',
              'migration_get_progress')
    def test_migration_start_get_share_exception(self, method):
        share = test_data.share
        url = reverse('horizon:admin:shares:' + method, args=[share.id])

        self.mock_object(
            api_manila, "share_get", mock.Mock(side_effect=Exception('fake')))
        self.mock_object(api_manila, method)

        res = self.client.get(url)

        api_manila.share_get.assert_called_once_with(mock.ANY, share.id)
        self.assertFalse(getattr(api_manila, method).called)

        self.assertEqual(res.status_code, 302)
        self.assertTemplateNotUsed(res, 'admin/shares/' + method + '.html')

    @ddt.data({'method': 'migration_complete', 'exc': None},
              {'method': 'migration_complete', 'exc': Exception('fake')},
              {'method': 'migration_cancel', 'exc': None},
              {'method': 'migration_cancel', 'exc': Exception('fake')},
              {'method': 'migration_get_progress',
               'exc': {'response': 200, 'total_progress': 25}},
              {'method': 'migration_get_progress', 'exc': Exception('fake')})
    @ddt.unpack
    def test_migration_forms_post(self, exc, method):
        share = test_data.share
        url = reverse('horizon:admin:shares:' + method,
                      args=[share.id])
        formData = {
            'share_id': share.id,
            'name': share.name,
        }

        self.mock_object(
            api_manila, "share_get", mock.Mock(return_value=share))
        self.mock_object(api_manila, method, mock.Mock(
            side_effect=exc))

        res = self.client.post(url, formData)

        api_manila.share_get.assert_called_once_with(mock.ANY, share.id)
        getattr(api_manila, method).assert_called_once_with(
            mock.ANY, formData['share_id'])

        status_code = 200 if exc else 302
        self.assertEqual(res.status_code, status_code)
        if not exc:
            self.assertTemplateNotUsed(
                res, 'admin/shares/' + method + '.html')
            self.assertRedirectsNoFollow(res, INDEX_URL)
        else:
            self.assertTemplateUsed(
                res, 'admin/shares/' + method + '.html')



# Copyright (c) 2017 Mirantis, Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from django.urls import reverse
from unittest import mock

from manila_ui.api import manila as api_manila
from manila_ui.tests.dashboards.project import test_data
from manila_ui.tests import helpers as test

INDEX_URL = reverse('horizon:admin:share_group_types:index')


class ShareGroupTypeTests(test.BaseAdminViewTests):

    def setUp(self):
        super(self.__class__, self).setUp()
        self.share_group_type = test_data.share_group_type
        self.share_group_type_alt = test_data.share_group_type_alt
        self.share_group_type_p = test_data.share_group_type_private
        self.url = reverse('horizon:admin:share_group_types:update',
                           args=[self.share_group_type.id])
        self.mock_object(api_manila, "share_group_type_get",
                         mock.Mock(return_value=self.share_group_type))

    def test_create_share_group_type(self):
        url = reverse('horizon:admin:share_group_types:create')
        data = {
            'method': 'CreateShareGroupTypeForm',
            'is_public': True,
            'name': 'my_share_group_type',
            'share_types': ['foo'],
            'group_specs': 'key=value',
        }
        self.mock_object(
            api_manila, "share_group_type_create",
            mock.Mock(return_value=type(
                'ShareGroupType',
                (object, ),
                {'id': 'sgt_id', 'name': 'sgt_name'})))
        self.mock_object(api_manila, "share_group_type_set_specs")
        self.mock_object(
            api_manila, "share_type_list",
            mock.Mock(return_value=[type(
                'ShareType',
                (object, ),
                {'id': s_id, 'name': s_id + '_name'})
                for s_id in ('foo', 'bar')
            ]))

        res = self.client.post(url, data)

        api_manila.share_group_type_create.assert_called_once_with(
            mock.ANY, data['name'],
            is_public=data['is_public'],
            share_types=['foo'])
        api_manila.share_type_list.assert_called_once_with(mock.ANY)
        api_manila.share_group_type_set_specs.assert_called_once_with(
            mock.ANY, 'sgt_id', {'key': 'value'})
        self.assertRedirectsNoFollow(res, INDEX_URL)

    def test_update_share_group_type_get(self):
        res = self.client.get(self.url)

        api_manila.share_group_type_get.assert_called_once_with(
            mock.ANY, self.share_group_type.id)
        self.assertNoMessages()
        self.assertTemplateUsed(res, 'admin/share_group_types/update.html')

    def test_update_share_group_type_post(self):
        data = {'group_specs': 'foo=bar'}
        form_data = {'group_specs': {'foo': 'bar'}}
        self.mock_object(api_manila, "share_group_type_set_specs")

        res = self.client.post(self.url, data)

        api_manila.share_group_type_set_specs.assert_called_once_with(
            mock.ANY, self.share_group_type.id, form_data['group_specs'])
        self.assertRedirectsNoFollow(res, INDEX_URL)

    def test_delete_share_group_type(self):
        data = {'action': 'share_group_types__delete__%s' %
                self.share_group_type_alt.id}
        self.mock_object(api_manila, "share_group_type_delete")
        self.mock_object(
            api_manila, "share_group_type_list",
            mock.Mock(return_value=[self.share_group_type_alt]))
        self.mock_object(
            api_manila, "share_type_list",
            mock.Mock(return_value=[test_data.share_type_alt]))

        res = self.client.post(INDEX_URL, data)

        api_manila.share_group_type_delete.assert_called_once_with(
            mock.ANY, self.share_group_type_alt.id)
        api_manila.share_group_type_list.assert_called_once_with(
            mock.ANY)
        self.assertRedirectsNoFollow(res, INDEX_URL)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from manila_ui.dashboards.utils import transform_dashed_name as tdn
from manila_ui.tests import helpers as test


class UtilsTests(test.TestCase):

    def test_no_transform(self):
        inputs = ['foo',
                  'bar01',
                  'baz_01']
        res = [tdn(x) for x in inputs]
        self.assertEqual(inputs, res)

    def test_transform(self):
        inputs = ['Foo',
                  'Bar01',
                  'baz-01']
        exp_res = ['izxw6___',
                   'ijqxembr',
                   'mjqxuljqge______']
        res = [tdn(x) for x in inputs]
        self.assertEqual(res, exp_res)

    def test_undo_transform(self):
        inputs = ['izxw6___',
                  'ijqxembr',
                  'mjqxuljqge______']
        exp_res = ['Foo',
                   'Bar01',
                   'baz-01']
        res = [tdn(x) for x in inputs]
        self.assertEqual(res, exp_res)



# Copyright 2012 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2012 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from horizon import exceptions as horizon_exceptions
from openstack_dashboard import api
from unittest import mock

from manila_ui.api import manila as api_manila
from manila_ui.tests.dashboards.project import test_data
from manila_ui.tests import helpers as test

INDEX_URL = reverse('horizon:project:security_services:index')


class SecurityServicesViewTests(test.TestCase):

    def test_create_security_service(self):
        sec_service = test_data.sec_service
        formData = {
            'name': 'new_sec_service',
            'description': 'This is test security service',
            'method': 'CreateForm',
            'dns_ip': '1.2.3.4',
            'ou': 'someOU',
            'user': 'SomeUser',
            'password': 'safepass',
            'confirm_password': 'safepass',
            'type': 'ldap',
            'domain': 'TEST',
            'server': 'testserver',
        }
        url = reverse(
            'horizon:project:security_services:security_service_create')
        self.mock_object(
            api_manila, "security_service_create",
            mock.Mock(return_value=sec_service))

        res = self.client.post(url, formData)

        del formData['method']
        del formData['confirm_password']
        api_manila.security_service_create.assert_called_with(
            mock.ANY, **formData)
        self.assertRedirectsNoFollow(res, INDEX_URL)

    def test_delete_security_service(self):
        security_service = test_data.sec_service
        formData = {
            'action': 'security_services__delete__%s' % security_service.id,
        }
        self.mock_object(api_manila, "security_service_delete")
        self.mock_object(
            api_manila, "security_service_list",
            mock.Mock(return_value=[security_service]))

        res = self.client.post(INDEX_URL, formData)

        api_manila.security_service_list.assert_called_with(mock.ANY)
        api_manila.security_service_delete.assert_called_with(
            mock.ANY, security_service.id)
        self.assertRedirectsNoFollow(res, INDEX_URL)

    def test_detail_view(self):
        sec_service = test_data.sec_service
        self.mock_object(
            api_manila, "security_service_get",
            mock.Mock(return_value=sec_service))
        url = reverse(
            'horizon:project:security_services:security_service_detail',
            args=[sec_service.id])

        res = self.client.get(url)

        self.assertContains(res, "<h1>Security Service Details: %s</h1>"
                                 % sec_service.name,
                            1, 200)
        self.assertContains(res, "<dd>%s</dd>" % sec_service.name, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % sec_service.id, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % sec_service.user, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % sec_service.server, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % sec_service.dns_ip, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % sec_service.ou, 1, 200)
        self.assertContains(res, "<dd>%s</dd>" % sec_service.domain, 1, 200)
        self.assertNoMessages()
        api_manila.security_service_get.assert_called_once_with(
            mock.ANY, sec_service.id)

    def test_detail_view_with_exception(self):
        url = reverse(
            'horizon:project:security_services:security_service_detail',
            args=[test_data.sec_service.id])
        self.mock_object(
            api_manila, "security_service_get",
            mock.Mock(side_effect=horizon_exceptions.NotFound(404)))

        res = self.client.get(url)

        self.assertRedirectsNoFollow(res, INDEX_URL)
        api_manila.security_service_get.assert_called_once_with(
            mock.ANY, test_data.sec_service.id)

    def test_update_security_service_get(self):
        sec_service = test_data.sec_service
        url = reverse(
            'horizon:project:security_services:security_service_update',
            args=[sec_service.id])
        self.mock_object(
            api_manila, "security_service_get",
            mock.Mock(return_value=sec_service))
        self.mock_object(
            api.neutron, "is_service_enabled", mock.Mock(return_value=[True]))

        res = self.client.get(url)

        self.assertNoMessages()
        self.assertTemplateUsed(
            res, 'project/security_services/update.html')
        api_manila.security_service_get.assert_called_once_with(
            mock.ANY, sec_service.id)

    def test_update_security_service_post(self):
        sec_service = test_data.sec_service
        url = reverse(
            'horizon:project:security_services:security_service_update',
            args=[sec_service.id])
        formData = {
            'method': 'UpdateForm',
            'name': sec_service.name,
            'description': sec_service.description,
            'dns_ip': sec_service.dns_ip,
            'ou': sec_service.ou,
            'server': sec_service.server,
            'domain': sec_service.domain,
            'password': sec_service.password,
            'confirm_password': sec_service.password,
            'user': sec_service.user,
        }
        self.mock_object(api_manila, "security_service_update")
        self.mock_object(
            api_manila, "security_service_get",
            mock.Mock(return_value=sec_service))

        res = self.client.post(url, formData)

        self.assertRedirectsNoFollow(res, INDEX_URL)
        api_manila.security_service_get.assert_called_once_with(
            mock.ANY, sec_service.id)
        api_manila.security_service_update.assert_called_once_with(
            mock.ANY,
            sec_service.id,
            name=formData['name'],
            description=formData['description'],
            dns_ip=formData['dns_ip'],
            ou=formData['ou'],
            server=formData['server'],
            domain=formData['domain'],
            password=formData['password'],
            user=formData['user'])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import workflows

from manila_ui.api import manila


class UpdateShareNetworkInfoAction(workflows.Action):
    name = forms.CharField(label=_("Name"),
                           max_length=255)

    description = forms.CharField(label=_("Description"),
                                  max_length=255,
                                  required=False)

    def __init__(self, request, *args, **kwargs):
        super(UpdateShareNetworkInfoAction, self).__init__(
            request, *args, **kwargs)
        share_network = manila.share_network_get(request,
                                                 self.initial['id'])
        self.fields['name'].initial = share_network.name
        self.fields['description'].initial = share_network.description

    class Meta(object):
        name = _("Share Network Info")
        help_text = _("From here you can update share network info. ")
        slug = "update-share_network_info"

    def clean(self):
        cleaned_data = super(UpdateShareNetworkInfoAction, self).clean()
        return cleaned_data


class UpdateShareNetworkInfoStep(workflows.Step):
    action_class = UpdateShareNetworkInfoAction
    contributes = ("description",
                   "name")


class AddSecurityServiceAction(workflows.MembershipAction):
    def __init__(self, request, *args, **kwargs):
        super(AddSecurityServiceAction, self).__init__(request,
                                                       *args,
                                                       **kwargs)
        err_msg = _('Unable to get the security services hosts')

        default_role_field_name = self.get_default_role_field_name()
        self.fields[default_role_field_name] = forms.CharField(required=False)
        self.fields[default_role_field_name].initial = 'member'

        field_name = self.get_member_field_name('member')
        self.fields[field_name] = forms.MultipleChoiceField(required=False)

        share_network_id = self.initial['id']
        security_services = manila.share_network_security_service_list(
            request, share_network_id)
        sec_services_initial = [sec_service.id for sec_service
                                in security_services]
        sec_services = []
        try:
            sec_services = manila.security_service_list(request)
        except Exception:
            exceptions.handle(request, err_msg)

        sec_services_choices = [(sec_service.id,
                                 sec_service.name or sec_service.id)
                                for sec_service in sec_services]
        self.fields[field_name].choices = sec_services_choices
        self.fields[field_name].initial = sec_services_initial

    class Meta(object):
        name = _("Security services within share network")
        slug = "add_security_service"


class AddSecurityServiceStep(workflows.UpdateMembersStep):
    action_class = AddSecurityServiceAction
    help_text = _("Add security services to share network.")
    available_list_title = _("Available security services")
    members_list_title = _("Selected security services")
    no_available_text = _("No security services found.")
    no_members_text = _("No security services selected.")
    show_roles = False
    depends_on = ("id",)
    contributes = ("security_services",)

    def contribute(self, data, context):
        if data:
            member_field_name = self.get_member_field_name('member')
            context['security_service'] = data.get(member_field_name, [])
        return context


class UpdateShareNetworkWorkflow(workflows.Workflow):
    slug = "update_share_network"
    name = _("Update Share Network")
    finalize_button_name = _("Update Share Network")
    success_message = _('Updated share network "%s".')
    failure_message = _('Unable to update share network "%s".')
    success_url = 'horizon:project:share_networks:index'
    default_steps = (UpdateShareNetworkInfoStep, AddSecurityServiceStep)

    def format_status_message(self, message):
        return message % self.context['name']

    def handle(self, request, context):
        try:
            manila.share_network_update(request, context['id'],
                                        description=context['description'],
                                        name=context['name'])
            sec_services = manila.security_service_list(request, search_opts={
                'share_network_id': context['id']})
            sec_services_old = set([sec_service.id
                                   for sec_service in sec_services])
            sec_services_new = set(context['security_service'])
            for sec_service in sec_services_new - sec_services_old:
                manila.share_network_security_service_add(request,
                                                          context['id'],
                                                          sec_service)
            for sec_service in sec_services_old - sec_services_new:
                manila.share_network_security_service_remove(request,
                                                             context['id'],
                                                             sec_service)
            return True
        except Exception:
            exceptions.handle(request, _('Unable to update share network.'))
            return False



#!/usr/bin/env python

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

#
# Starting with Django 1.7, when a django app is loaded, it is assigned a
# default label containing the portion of the application name after the last
# period, and this name has to be globally unique.  When horizon project
# dashboard, openstack_dashboard.dashboards.project, is loaded, it is assigned
# the label 'project'.  But when the manila dashboard
# manila_ui.dashboards.project is loaded, it label will conflict with
# horizon's.  Therefore this AppConfig class exists merely to specify a unique
# configuration label and avoid this conflict.
#

from django import apps


class Config(apps.AppConfig):
    name = 'manila_ui.dashboards.project'
    label = 'manila_project'



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.urls import re_path

from manila_ui.dashboards.admin.share_snapshots import views


urlpatterns = [
    re_path(
        r'^$',
        views.ShareSnapshotsView.as_view(),
        name='index'),
    re_path(
        r'^(?P<snapshot_id>[^/]+)$',
        views.ShareSnapshotDetailView.as_view(),
        name='share_snapshot_detail'),
]



# (c) Copyright 2019 SUSE LLC
# All rights reserved.

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.test.utils import override_settings
from django.urls import reverse

from horizon.workflows import views
from openstack_dashboard.api import base
from openstack_dashboard.usage import quotas

from manila_ui.api import manila as manila_api
from manila_ui.tests import helpers as manila_base_test


class UpdateShareQuotaWorkflowTests(manila_base_test.BaseAdminViewTests):

    def setUp(self):
        super(UpdateShareQuotaWorkflowTests, self).setUp()
        self.id = "fake_id"

        # mock functions in manila.py
        self.mock_tenant_quota_get = (
            self.mock_object(manila_api, "tenant_quota_get"))
        self.mock_tenant_quota_get.return_value = (
            self._get_mock_share_quota_data())

        # mock functions in quotas.py
        self.mock_object(quotas, "get_tenant_quota_data")
        self.mock_object(quotas, "get_disabled_quotas").return_value = set()

    def _get_mock_share_quota_data(self):
        quota_data = {
            'shares': 24, 'gigabytes': 333, 'snapshots': 14,
            'snapshot_gigabytes': 444, 'share_networks': 14
        }

        return base.QuotaSet(quota_data)

    @override_settings(EXTRA_STEPS={
        'openstack_dashboard.dashboards.identity.projects.\
         workflows.UpdateQuota': (
            'manila_ui.dashboards.identity.projects.workflows.UpdateShareQuota'
        )}
    )
    def test_tenant_quota_get(self):
        tenant_id = 'fake_tenant_id'
        url = reverse('horizon:identity:projects:update_quotas',
                      args=[tenant_id])
        res = self.client.get(url)

        workflow = res.context['workflow']
        self.assertTemplateUsed(res, views.WorkflowView.template_name)
        step = workflow.get_step("update_share_quotas")

        # Check step UI fields got the fields converted and data correct
        self.assertEqual(step.action.initial['project_id'], tenant_id)
        self.assertEqual(step.action.initial['shares'], 24)
        self.assertEqual(step.action.initial['share_gigabytes'], 333)
        self.assertEqual(step.action.initial['share_snapshot_gigabytes'], 444)
        self.assertEqual(step.action.initial['share_snapshots'], 14)
        self.assertEqual(step.action.initial['share_networks'], 14)
